@@ -22,21 +22,28 @@ import static org.hamcrest.Matchers.equalTo;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.io.ByteArrayOutputStream;
 import java.io.DataOutputStream;
 import java.io.IOException;
 import java.util.Arrays;
+import java.util.Map;
 
 import javax.jms.BytesMessage;
+import javax.jms.CompletionListener;
 import javax.jms.Connection;
+import javax.jms.DeliveryMode;
 import javax.jms.JMSException;
 import javax.jms.Message;
 import javax.jms.MessageConsumer;
+import javax.jms.MessageFormatException;
+import javax.jms.MessageNotWriteableException;
 import javax.jms.MessageProducer;
 import javax.jms.Queue;
 import javax.jms.Session;
 
+import org.apache.qpid.jms.JmsConnection;
 import org.apache.qpid.jms.provider.amqp.message.AmqpMessageSupport;
 import org.apache.qpid.jms.test.QpidJmsTestCase;
 import org.apache.qpid.jms.test.testpeer.TestAmqpPeer;
@@ -87,6 +94,11 @@ public class BytesMessageIntegrationTest extends QpidJmsTestCase {
             message.writeBytes(content);
 
             producer.send(message);
+
+            testPeer.expectClose();
+            connection.close();
+
+            testPeer.waitForAllHandlersToComplete(3000);
         }
     }
 
@@ -148,6 +160,11 @@ public class BytesMessageIntegrationTest extends QpidJmsTestCase {
             int readBytes = bytesMessage.readBytes(recievedContent);
             assertEquals(recievedContent.length, readBytes);
             assertTrue(Arrays.equals(expectedContent, recievedContent));
+
+            testPeer.expectClose();
+            connection.close();
+
+            testPeer.waitForAllHandlersToComplete(3000);
         }
     }
 
@@ -157,6 +174,8 @@ public class BytesMessageIntegrationTest extends QpidJmsTestCase {
      * gives the expected data values when read, and when reset and left mid-stream before being
      * resent that it results in the expected AMQP data body section and properties content type
      * being received by the test peer.
+     *
+     * @throws Exception if an error occurs during the test.
      */
     @Test(timeout = 20000)
     public void testReceiveBytesMessageAndResendAfterResetAndPartialRead() throws Exception {
@@ -255,6 +274,71 @@ public class BytesMessageIntegrationTest extends QpidJmsTestCase {
 
             producer.send(receivedBytesMessage);
 
+            testPeer.expectClose();
+            connection.close();
+
+            testPeer.waitForAllHandlersToComplete(3000);
+        }
+    }
+
+    @Test(timeout = 20000)
+    public void testGetBodyBytesMessageFailsWhenWrongTypeRequested() throws Exception {
+        try (TestAmqpPeer testPeer = new TestAmqpPeer();) {
+            Connection connection = testFixture.establishConnecton(testPeer);
+            connection.start();
+
+            testPeer.expectBegin();
+
+            Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            Queue queue = session.createQueue("myQueue");
+
+            PropertiesDescribedType properties = new PropertiesDescribedType();
+            properties.setContentType(Symbol.valueOf(AmqpMessageSupport.OCTET_STREAM_CONTENT_TYPE));
+
+            MessageAnnotationsDescribedType msgAnnotations = null;
+            msgAnnotations = new MessageAnnotationsDescribedType();
+            msgAnnotations.setSymbolKeyedAnnotation(AmqpMessageSupport.JMS_MSG_TYPE, AmqpMessageSupport.JMS_BYTES_MESSAGE);
+
+            final byte[] expectedContent = "expectedContent".getBytes();
+            DescribedType dataContent = new DataDescribedType(new Binary(expectedContent));
+
+            testPeer.expectReceiverAttach();
+            testPeer.expectLinkFlowRespondWithTransfer(null, msgAnnotations, properties, null, dataContent, 2);
+            testPeer.expectDispositionThatIsAcceptedAndSettled();
+            testPeer.expectDispositionThatIsAcceptedAndSettled();
+
+            MessageConsumer messageConsumer = session.createConsumer(queue);
+
+            Message readMsg1 = messageConsumer.receive(1000);
+            Message readMsg2 = messageConsumer.receive(1000);
+
+            testPeer.waitForAllHandlersToComplete(3000);
+
+            assertNotNull(readMsg1);
+            assertNotNull(readMsg2);
+
+            try {
+                readMsg1.getBody(String.class);
+                fail("Should have thrown MessageFormatException");
+            } catch (MessageFormatException mfe) {
+            }
+
+            try {
+                readMsg2.getBody(Map.class);
+                fail("Should have thrown MessageFormatException");
+            } catch (MessageFormatException mfe) {
+            }
+
+            byte[] received1 = readMsg1.getBody(byte[].class);
+            byte[] received2 = (byte[]) readMsg2.getBody(Object.class);
+
+            assertTrue(Arrays.equals(expectedContent, received1));
+            assertTrue(Arrays.equals(expectedContent, received2));
+
+            testPeer.expectClose();
+
+            connection.close();
+
             testPeer.waitForAllHandlersToComplete(3000);
         }
     }
@@ -265,6 +349,8 @@ public class BytesMessageIntegrationTest extends QpidJmsTestCase {
      * expected data values when read, and when sent to the test peer it results in an
      * AMQP message containing a data body section and content type of
      * {@link AmqpMessageSupport#OCTET_STREAM_CONTENT_TYPE}
+     *
+     * @throws Exception if an error occurs during the test.
      */
     @Test(timeout = 20000)
     public void testReceiveBytesMessageWithAmqpValueAndResendResultsInData() throws Exception {
@@ -360,7 +446,219 @@ public class BytesMessageIntegrationTest extends QpidJmsTestCase {
 
             producer.send(receivedBytesMessage);
 
+            testPeer.expectClose();
+            connection.close();
+
             testPeer.waitForAllHandlersToComplete(3000);
         }
     }
+
+    @Test(timeout = 20000)
+    public void testAsyncSendDoesNotMarksBytesMessageReadOnly() throws Exception {
+        try(TestAmqpPeer testPeer = new TestAmqpPeer();) {
+            JmsConnection connection = (JmsConnection) testFixture.establishConnecton(testPeer);
+            connection.setSendTimeout(15000);
+
+            testPeer.expectBegin();
+
+            Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            String queueName = "myQueue";
+            Queue queue = session.createQueue(queueName);
+
+            BytesMessage message = session.createBytesMessage();
+            TransferPayloadCompositeMatcher messageMatcher = new TransferPayloadCompositeMatcher();
+
+            // Expect the producer to attach and grant it some credit, it should send
+            // a transfer which we will not send any response so that we can check that
+            // the inflight message is read-only
+            testPeer.expectSenderAttach();
+            testPeer.expectTransferButDoNotRespond(messageMatcher);
+            testPeer.expectClose();
+
+            MessageProducer producer = session.createProducer(queue);
+            producer.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
+
+            try {
+                producer.send(message);
+            } catch (Throwable error) {
+                fail("Send should not fail for async send.");
+            }
+
+            try {
+                message.setJMSCorrelationID("test");
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSCorrelationIDAsBytes(new byte[]{});
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSDeliveryMode(DeliveryMode.PERSISTENT);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSDestination(queue);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSExpiration(0);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSMessageID(queueName);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSPriority(0);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSRedelivered(false);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSReplyTo(queue);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSTimestamp(0);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setJMSType(queueName);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.setStringProperty("test", "test");
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to set properties on inflight message");
+            }
+            try {
+                message.clearBody();
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to clear on inflight message");
+            }
+            try {
+                message.writeBoolean(true);
+            } catch (MessageNotWriteableException mnwe) {
+                fail("Should be able to write to on inflight message");
+            }
+
+            connection.close();
+
+            testPeer.waitForAllHandlersToComplete(1000);
+        }
+    }
+
+    @Test(timeout = 20000)
+    public void testAsyncCompletionSendMarksBytesMessageReadOnly() throws Exception {
+        try(TestAmqpPeer testPeer = new TestAmqpPeer();) {
+            JmsConnection connection = (JmsConnection) testFixture.establishConnecton(testPeer);
+            connection.setSendTimeout(15000);
+
+            testPeer.expectBegin();
+
+            Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
+            String queueName = "myQueue";
+            Queue queue = session.createQueue(queueName);
+
+            BytesMessage message = session.createBytesMessage();
+            TransferPayloadCompositeMatcher messageMatcher = new TransferPayloadCompositeMatcher();
+
+            // Expect the producer to attach and grant it some credit, it should send
+            // a transfer which we will not send any response so that we can check that
+            // the inflight message is read-only
+            testPeer.expectSenderAttach();
+            testPeer.expectTransferButDoNotRespond(messageMatcher);
+            testPeer.expectClose();
+
+            MessageProducer producer = session.createProducer(queue);
+            TestJmsCompletionListener listener = new TestJmsCompletionListener();
+
+            try {
+                producer.send(message, listener);
+            } catch (Throwable error) {
+                fail("Send should not fail for async.");
+            }
+
+            try {
+                message.setJMSCorrelationID("test");
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSCorrelationIDAsBytes(new byte[]{});
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSDeliveryMode(DeliveryMode.PERSISTENT);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSDestination(queue);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSExpiration(0);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSMessageID(queueName);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSPriority(0);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSRedelivered(false);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSReplyTo(queue);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSTimestamp(0);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setJMSType(queueName);
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.setStringProperty("test", "test");
+                fail("Should not be able to set properties on inflight message");
+            } catch (MessageNotWriteableException mnwe) {}
+            try {
+                message.writeBoolean(true);
+                fail("Message should not be writable after a send.");
+            } catch (MessageNotWriteableException mnwe) {}
+
+            connection.close();
+
+            testPeer.waitForAllHandlersToComplete(1000);
+        }
+    }
+
+    private class TestJmsCompletionListener implements CompletionListener {
+
+        @Override
+        public void onCompletion(Message message) {
+        }
+
+        @Override
+        public void onException(Message message, Exception exception) {
+        }
+    }
 }
